@@ -8,17 +8,17 @@ import click
 import spyctl.config.configs as cfgs
 import spyctl.config.secrets as s
 import spyctl.spyctl_lib as lib
-import spyctl.subcommands.get as g
-import spyctl.subcommands.create as c
-import spyctl.subcommands.merge as m
-import spyctl.subcommands.diff as d
-from spyctl.subcommands.apply import handle_apply
-from spyctl.subcommands.delete import handle_delete
+import spyctl.commands.get as g
+import spyctl.commands.create as c
+import spyctl.commands.merge as m
+import spyctl.commands.diff as d
+import spyctl.cli as cli
+from spyctl.commands.apply import handle_apply
+from spyctl.commands.delete import handle_delete
 
 MAIN_EPILOG = (
     'Use "spyctl <command> --help" for more information about a given command'
 )
-DEFAULT_API_URL = "https://api.spyderbat.com"
 DEFAULT_START_TIME = 1614811600
 
 # ----------------------------------------------------------------- #
@@ -28,14 +28,13 @@ DEFAULT_START_TIME = 1614811600
 
 @click.group(cls=lib.CustomGroup, epilog=MAIN_EPILOG)
 @click.help_option("-h", "--help", hidden=True)
-@click.version_option()
+@click.version_option(None, "-v", "--version", prog_name="Spyctl", hidden=True)
 @click.pass_context
 def main(ctx: click.Context):
     """spyctl displays and controls resources within your Spyderbat
     environment
     """
     cfgs.load_config()
-    lib.add_to_cmd_tree(lib.APP_NAME)
 
 
 # ----------------------------------------------------------------- #
@@ -67,7 +66,6 @@ def apply(filename):
 @click.pass_context
 def config(ctx: click.Context):
     """Modify spyctl config files."""
-    lib.add_to_cmd_tree(ctx.command.name)
 
 
 @config.command("delete-context", cls=lib.CustomCommand, epilog=MAIN_EPILOG)
@@ -80,12 +78,40 @@ def config(ctx: click.Context):
     help="When operating within a spyctl workspace, this forces a change to"
     " the global spyctl config.",
 )
+@click.option(
+    "-y",
+    "--yes",
+    "--assume-yes",
+    is_flag=True,
+    help='Automatic yes to prompts; assume "yes" as answer to all prompts and'
+    " run non-interactively.",
+)
 @click.argument("name")
-def delete_context(name, force_global):
+def delete_context(name, force_global, yes=False):
     """Delete the specified context from a spyctl configuration file."""
+    if yes:
+        cli.set_yes_option()
     cfgs.delete_context(name, force_global)
 
 
+@config.command("delete-apisecret", cls=lib.CustomCommand, epilog=MAIN_EPILOG)
+@click.help_option("-h", "--help", hidden=True)
+@click.option(
+    "-y",
+    "--yes",
+    "--assume-yes",
+    is_flag=True,
+    help='Automatic yes to prompts; assume "yes" as answer to all prompts and'
+    " run non-interactively.",
+)
+@click.argument("name", type=s.SecretsParam())
+def delete_apisecret(name, yes=False):
+    """Delete the specified context from a spyctl configuration file."""
+    if yes:
+        cli.set_yes_option()
+    s.delete_secret(name)
+
+
 @config.command("current-context", cls=lib.CustomCommand, epilog=MAIN_EPILOG)
 @click.help_option("-h", "--help", hidden=True)
 @click.option(
@@ -103,7 +129,7 @@ def current_context(force_global):
 
 @config.command("get-contexts", cls=lib.CustomCommand, epilog=MAIN_EPILOG)
 @click.help_option("-h", "--help", hidden=True)
-@click.argument("name", required=False)
+@click.argument("name", required=False, type=cfgs.ContextsParam())
 @click.option(
     "-o",
     "--output",
@@ -122,10 +148,25 @@ def current_context(force_global):
 )
 def get_contexts(force_global, output, name=None):
     """Describe one or many contexts"""
-    lib.add_to_cmd_tree(click.get_current_context().command.name)
     cfgs.get_contexts(name, force_global, output)
 
 
+@config.command("get-apisecrets", cls=lib.CustomCommand, epilog=MAIN_EPILOG)
+@click.help_option("-h", "--help", hidden=True)
+@click.argument("name", required=False, type=s.SecretsParam())
+@click.option(
+    "-o",
+    "--output",
+    default=lib.OUTPUT_DEFAULT,
+    type=click.Choice(
+        lib.OUTPUT_CHOICES + [lib.OUTPUT_WIDE], case_sensitive=False
+    ),
+)
+def get_api_secrets(output, name=None):
+    """Describe one or many apisecrets"""
+    s.handle_get_secrets(name, output)
+
+
 @config.command("set-context", cls=lib.CustomCommand, epilog=MAIN_EPILOG)
 @click.help_option("-h", "--help", hidden=True)
 @click.option(
@@ -195,13 +236,41 @@ def get_contexts(force_global, output, name=None):
 @click.option("-C", "--cgroup", help="Linux service cgroup.", metavar="")
 @click.argument("name")
 def set_context(name, secret, force_global, use_ctx, **context):
-    """Set a context entry in a spyctl configuration file."""
+    """Set a context entry in a spyctl configuration file, or update an
+    existing one.
+    """
     context = {
         key: value for key, value in context.items() if value is not None
     }
     cfgs.set_context(name, secret, force_global, use_ctx, **context)
 
 
+@config.command("set-apisecret", cls=lib.CustomCommand, epilog=MAIN_EPILOG)
+@click.help_option("-h", "--help", hidden=True)
+@click.argument("name", required=False)
+@click.option(
+    "-k",
+    "--apikey",
+    "--api-key",
+    "api_key",
+    help="API key generated via the Spyderbat UI",
+    metavar="",
+)
+@click.option(
+    "-u",
+    "--apiurl",
+    "--api-url",
+    "api_url",
+    help=f"URL target for api queries. Default: {lib.DEFAULT_API_URL}",
+    metavar="",
+)
+def set_apisecrets(api_key=None, api_url=None, name=None):
+    """
+    Set a new entry in the spyctl secrets file, or update an existing one.
+    """
+    s.set_secret(name, api_url, api_key)
+
+
 @config.command("use-context", cls=lib.CustomCommand, epilog=MAIN_EPILOG)
 @click.help_option("-h", "--help", hidden=True)
 @click.option(
@@ -212,7 +281,7 @@ def set_context(name, secret, force_global, use_ctx, **context):
     help="When operating within a spyctl workspace, this forces a change to"
     " the global spyctl config.",
 )
-@click.argument("name")
+@click.argument("name", type=cfgs.ContextsParam())
 def use_context(name, force_global):
     """Set the current-context in a spyctl configuration file."""
     cfgs.use_context(name, force_global)
@@ -303,68 +372,6 @@ def create_policy(filename, output):
     c.handle_create_policy(filename, output)
 
 
-@create.group("secret", cls=lib.CustomSubGroup, epilog=MAIN_EPILOG)
-@click.help_option("-h", "--help", hidden=True)
-def create_secret():
-    """Create a Secret object from the command line."""
-    pass
-
-
-@create_secret.command("apicfg", cls=lib.CustomCommand, epilog=MAIN_EPILOG)
-@click.help_option("-h", "--help", hidden=True)
-@click.option(
-    "-k",
-    "--apikey",
-    "--api-key",
-    "api_key",
-    help="API key generated via the Spyderbat UI, base64 encoded."
-    " Use 'echo -n <apikey> | base64 -w 1000'",
-    metavar="",
-)
-@click.option(
-    "-u",
-    "--apiurl",
-    "--api-url",
-    "api_url",
-    help=f"URL target for api queries. Default: {DEFAULT_API_URL}",
-    default=DEFAULT_API_URL,
-    metavar="",
-)
-@click.argument("name")
-def create_apicfg_secret(name, api_key, api_url):
-    """Create an apicfg secret. spyctl requires an api secret be applied to a
-    context in order to run successful api queries.
-    """
-    s.create_secret(
-        name,
-        s.S_TYPE_APICFG,
-        data={lib.API_KEY_FIELD: api_key},
-        string_data={lib.API_URL_FIELD: api_url},
-    )
-
-
-@create_secret.command("opaque", cls=lib.CustomCommand, epilog=MAIN_EPILOG)
-@click.help_option("-h", "--help", hidden=True)
-@click.option(
-    "--data",
-    help="Key/value pairs, base64 encoded.",
-    metavar="",
-)
-@click.option(
-    "--string-data",
-    "string_data",
-    help="Key/value pairs, plain text.",
-    default=DEFAULT_API_URL,
-    metavar="",
-)
-@click.argument("name")
-def create_opaque_secret(name, data=None, string_data=None):
-    """Create an opaque secret. May contain an arbitrary amount of base64
-    encoded data and plaintext string data.
-    """
-    s.create_secret(name, s.S_TYPE_OPAQUE, data, string_data)
-
-
 # ----------------------------------------------------------------- #
 #                        Delete Subcommand                          #
 # ----------------------------------------------------------------- #
@@ -372,10 +379,20 @@ def create_opaque_secret(name, data=None, string_data=None):
 
 @main.command("delete", cls=lib.CustomCommand, epilog=MAIN_EPILOG)
 @click.help_option("-h", "--help", hidden=True)
-@click.argument("resource")
+@click.argument("resource", type=lib.DelResourcesParam())
 @click.argument("name_or_id")
-def delete(resource, name_or_id):
+@click.option(
+    "-y",
+    "--yes",
+    "--assume-yes",
+    is_flag=True,
+    help='Automatic yes to prompts; assume "yes" as answer to all prompts and'
+    " run non-interactively.",
+)
+def delete(resource, name_or_id, yes=False):
     """Delete resources by resource and names, or by resource and ids"""
+    if yes:
+        cli.set_yes_option()
     handle_delete(resource, name_or_id)
 
 
@@ -420,7 +437,7 @@ def diff(filename, with_file=None, latest=False):
 
 @main.group("get", cls=lib.CustomCommand, epilog=MAIN_EPILOG)
 @click.help_option("-h", "--help", hidden=True)
-@click.argument("resource")
+@click.argument("resource", type=lib.GetResourcesParam())
 @click.argument("name_or_id", required=False)
 @click.option(
     "--image",
@@ -480,20 +497,37 @@ def diff(filename, with_file=None, latest=False):
     default=lib.OUTPUT_DEFAULT,
     type=click.Choice(lib.OUTPUT_CHOICES, case_sensitive=False),
 )
+@click.option(
+    "-f",
+    "--filename",
+    help="Input file to create filters from. Used if you want to get resources"
+    " directly related to another resource. E.g. the Fingerprint Groups for a"
+    " given Baseline file.",
+    metavar="",
+    type=click.File(),
+)
 @click.option(
     "-l",
     "--latest",
-    help=f"Filename for resource. If there is a {lib.LATEST_TIMESTAMP_FIELD}"
-    " in the resources metadata field, the start time of the query is set to"
-    " that.",
-    metavar="",
+    help="Starting time of the query is set to the"
+    f" {lib.LATEST_TIMESTAMP_FIELD}"
+    f" field the input resource's {lib.METADATA_FIELD}."
+    " [Requires '--filename' option to be set]",
+    is_flag=True,
+)
+@click.option(
+    "-E",
+    "--exact",
+    "--exact-match",
+    is_flag=True,
+    help="Exact match for name or ID.",
 )
 @click.option(
     "-t",
     "--start-time",
     "st",
-    help="Start time of the query. Default is beginning of time.",
-    default="2h",
+    help="Start time of the query. Default is 24 hours ago.",
+    default="24h",
     type=lib.time_inp,
 )
 @click.option(
@@ -504,13 +538,32 @@ def diff(filename, with_file=None, latest=False):
     default=time.time(),
     type=lib.time_inp,
 )
-def get(resource, st, et, output, name_or_id=None, latest=None, **filters):
+def get(
+    resource,
+    st,
+    et,
+    output,
+    filename=None,
+    exact=False,
+    name_or_id=None,
+    latest=None,
+    **filters,
+):
     """Display one or many resources."""
     filters = {
         key: value for key, value in filters.items() if value is not None
     }
-    lib.add_to_cmd_tree(click.get_current_context().command.name)
-    g.handle_get(resource, name_or_id, st, et, latest, output, **filters)
+    g.handle_get(
+        resource,
+        name_or_id,
+        st,
+        et,
+        filename,
+        latest,
+        exact,
+        output,
+        **filters,
+    )
 
 
 # ----------------------------------------------------------------- #
